# Copyright 2020 NullConvergence
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This module mines commits and contains all related Neo4j queries"""

from graphrepo.miners.default import DefaultMiner


class CommitMiner(DefaultMiner):
  def __init__(self, *args, **kwargs):
    super().__init__(*args, **kwargs)

  def query(self, **kwargs):
    return self.node_matcher.match("Commit", **kwargs)

  def get_all(self):
    return self.node_matcher.match("Commit")
